////////////////////////////////////
//																//
// Array Class Template	  				//
//																//
//		Chungho Chau								//
//																//
////////////////////////////////////

#ifndef TEMPLATE_CLASS
#define TEMPLATE_CLASS

#include <iostream.h>
#include "Generic.h"
#define IncrementSize 1		// user defined array incremental size



template <class T>
class Array {
	friend ostream &operator<< (ostream &output, const Array<T> &other);

	public:

				Array();				
				~Array();
				Array(const Array<T> &other);

				void Add(T element);
				void Add(T *array, int number);

				void Add_Element(void);

				void Remove(int index);

				BOOL Search(T element) const;

				Array &operator= (const Array &other);

				BOOL operator== (const Array &other) const;

				BOOL operator!= (const Array &other) const;

				T &operator[] (int index);

				int Size(void) const
					{ return m_quantity; }


	private:
				T   *m_array;					// member array capable of many different Types
				int m_quantity;				// total current array size, mutiple of 5
				int m_point;					// Points to the next empty slot to be filled				
};


//--------------------------------------------------------------------------------
//	Constructor 
//-------------------------------------------------------------------------------- 
template <class T>
Array<T>::Array()
{
	m_array = new T [IncrementSize];
	m_quantity = IncrementSize;				// initially, each array contains a number of empty slots
	m_point=0;
	for (int i=0; i<m_quantity; ++i)  // initialize member array
		m_array[i] = T();
}

//--------------------------------------------------------------------------------
//	Destructor  
//-------------------------------------------------------------------------------- 
template <class T>
Array<T>::~Array()
{
	delete [] m_array;
}

//--------------------------------------------------------------------------------
//	Copy Constructor
//-------------------------------------------------------------------------------- 
template <class T>
Array<T>::Array(const Array<T> &other)
{
	m_quantity = other.m_quantity;
	m_point = other.m_point;
	m_array = new T [m_quantity+1];  // type T
	for (int i=0; i<m_quantity; ++i)
		{ m_array[i] = other.m_array[i]; }
}


//--------------------------------------------------------------------------------
//	Add Element into Array
//-------------------------------------------------------------------------------- 
template <class T>
void Array<T>::Add(T element)
{
	if (m_quantity < 1) 
		return;
	
	if ( m_point == m_quantity)
		Add_Element();

	m_array[m_point] = element;
	m_point++;	
}


//--------------------------------------------------------------------------------
//	Increase Array Size 
//-------------------------------------------------------------------------------- 
template <class T>
void Array<T>::Add_Element(void)
{
	T *temp = new T [m_quantity+1]; 
	for (int i=0; i<m_point; ++i)
		temp[i] = m_array[i];

	delete [] m_array;

	m_quantity += IncrementSize;
	m_array = new T [m_quantity+1];
		
	for (i=0; i<m_point; ++i)
		m_array[i] = temp[i];

	delete [] temp;
}


//--------------------------------------------------------------------------------
//	Add Entire Array into Member Array
//-------------------------------------------------------------------------------- 
template <class T>
void Array<T>::Add(T *array, int number)
{
	if (number < 1)
		return;

	while ( number > (m_quantity - m_point) )
		Add_Element();

	int temp = m_point;
	for (int i=temp; i<(temp + number); ++i)
	{
		m_array[i] = array[i-temp];
		m_point++;
	}
}

//--------------------------------------------------------------------------------
//	Remove an Element from the array 
//-------------------------------------------------------------------------------- 
template <class T>
void Array<T>::Remove(int index)
{
	for (int i=index; i<m_point; ++i)
		m_array[i-1] = m_array[i];

	--m_point;					// move back one slot
	m_array[i-1] = T();	// reset the last element 

}

//--------------------------------------------------------------------------------
//	Search the Array
//-------------------------------------------------------------------------------- 
template <class T>
BOOL Array<T>::Search(T element) const
{
	for (int i=0; i<m_point; ++i)
	{
		if ( m_array[i] == element )
			return TRUE;
	}

	return FALSE;
}


//--------------------------------------------------------------------------------
//	Overloading operator= 
//-------------------------------------------------------------------------------- 
template <class T>
Array<T> &Array<T>::operator= (const Array<T> &other)
{
	if (this == &other)				// compare the pointer !!
	  return *this;		

	if (m_array) delete [] m_array;
	
	m_quantity = other.m_quantity;
	m_point = other.m_point;
	m_array = new T [m_quantity+1];
	for (int i=0; i<m_quantity; ++i)
		{ m_array[i] = other.m_array[i]; }

	return *this;
}


//--------------------------------------------------------------------------------
//	Overloading operator== 
//-------------------------------------------------------------------------------- 
template <class T>
BOOL Array<T>::operator== (const Array<T> &other) const
{
	if (m_quantity != other.m_quantity)
		return FALSE;

	for (int i=0; i<m_quantity; ++i)
	{
		if ( !(m_array[i] == other.m_array[i]) )
			return FALSE;
	}

	return TRUE;
}


//--------------------------------------------------------------------------------
//	Overloading operator!= 
//-------------------------------------------------------------------------------- 
template <class T>
BOOL Array<T>::operator!= (const Array<T> &other) const
{
	if (*this == other)
		return FALSE;
	else 
		return TRUE;
}


//--------------------------------------------------------------------------------
//	Overloading operator[]
//-------------------------------------------------------------------------------- 
template <class T>
T &Array<T>::operator[] (int index)
{
	if ( (index < 1) || (index > m_quantity) )
		return m_array[0];

	return m_array[index-1];
}


//--------------------------------------------------------------------------------
//	User Define Output Format, overload op<<
//-------------------------------------------------------------------------------- 
template <class T>
ostream &operator<< (ostream &output, const Array<T> &other)
{
	if (other.m_quantity == 0)
		return output;
	output << '[';
	for (int i=0; i<other.m_point; ++i)
	{
		output << other.m_array[i];
		if (i<other.m_point-1)
			{cout << ",";}
	}
	output << ']';
	return output;
}


#endif