//////////////////////////////////////
// Array Template Header File				//
//																	//
//	with Implementation							//
//																	//
//////////////////////////////////////

#ifndef TEMPLATE_CLASS
#define TEMPLATE_CLASS

#include "Generic.h"
#include <iostream.h>

template <class T>
class Array{
	friend ostream &operator<< (ostream& output, const Array<T>& other);

	public:
		Array(int size=INCREMENT);  // INCREMENT is defined in Generic.h
		~Array();

		Array(const Array<T>& other);

		void Add(T element);
		void Add(T* array, int size);

		void AddElement(void);

		int GetSize(void) const {return m_location;}

		Array &operator= (const Array<T>& other);

		BOOL operator== (const Array<T>& other);

		BOOL operator!= (const Array<T>& other)
			{ return !(*this == other); }

		T &operator[] (int index);

		static int GetArrayCount() {return m_arrayCount;}

	private:
		T* m_array;	  // pointer to first array element
		int		m_location;	// current location of empty spot
		int		m_total;		// total number of array element
		static int m_arrayCount;	// number of array instantiated
};

// $$ static memeber data $$
template <class T>
int Array<T>::m_arrayCount=0;


// $$ Constructor $$
template <class T>
Array<T>::Array(int size)
{
	m_total = size;
	m_array = new T [m_total];
	m_location = 0;	
	m_arrayCount ++;
}


// $$ Destructor $$
template <class T>
Array<T>::~Array()
{
	delete [] m_array;
	m_arrayCount --;
}


// $$ Copy Constructor $$
template <class T>
Array<T>::Array(const Array<T>& other)
{
	m_array = new T [other.m_total];
	m_total = other.m_total;
	m_location = other.m_location;	

	for (int i=0; i<m_location; i++)
		m_array[i] = other.m_array[i];	

	m_arrayCount ++;
}
	

// $$ Add function to add individual element $$
template <class T>
void Array<T>::Add(T element)
{
	if (m_location == m_total)
		AddElement();

	m_array[m_location] = element;
	m_location ++;
}


// $$ Add function to add an entire array of same type $$
template <class T>
void Array<T>::Add(T* array, int size)
{
	if ( size<1 )
		return;

	while ( size > (m_total - m_location) )
		AddElement();

	for (int i=0; i< size; i++)
		m_array[i+m_location] = array[i];

	m_location += size;
}


// $$ Increase array element by size of variable INCREMENT $$
template <class T>
void Array<T>::AddElement(void)
{
	T* temp = new T [m_total];
	for (int i=0; i<m_location; i++)
		temp[i] = m_array[i];

	m_total += INCREMENT;	
	delete [] m_array;

	m_array = new T [m_total];

	for (i=0; i<m_location; i++)
		m_array[i] = temp[i];

	for (i=m_location; i<m_total; i++)
		m_array[i] = 0;

	delete [] temp;
}


// $$ operator= overloading $$
template <class T>
Array<T> &Array<T>::operator= (const Array<T>& other)
{
	if (this == &other)				// compare the pointer !!
	  return *this;		

	if (m_array)
		delete [] m_array;

	m_array = new T [other.m_total];
	m_total = other.m_total;
	m_location = other.m_location;

	for (int i=0; i<m_location; i++)
		m_array[i] = other.m_array[i];

	return *this;
}


// $$ operator== overloading $$
template <class T>
BOOL Array<T>::operator== (const Array<T>& other)
{
	if (m_location != other.m_location)
		return FALSE;

	for (int i=0; i<m_location; i++)
	{
		if ( m_array[i] != other.m_array[i] )
			return FALSE;
	}

	return TRUE; 
}


// $$ operator[] overloading $$
template <class T>
T &Array<T>::operator[] (int index)
{
	if ( (index<0) || (index>=m_location) )
		return m_array[0];
	else
		return m_array[index-1];
}


// $$ operator<< overloading $$
template <class T>
ostream &operator<< (ostream& output, const Array<T>& other)
{
	if (other.m_location == 0)
		return output << "[]";

	output << '[';
	for (int i=0; i<other.m_location; i++)
	{
		output << other.m_array[i];
		if (i != other.m_location-1)
			output << ',';
	}

	output << ']';
	return output;
}


#endif